/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * File created on Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include "interface.h"
#include "eprintf.h"

Args *getArgs(int argc, char *argv[]){
  Args *args;
  int c;
  char *optString = "s:chv";
  
  args = (Args *)emalloc(sizeof(Args));
  args->s = NULL;
  args->e = 0;
  args->c = 0;
  args->h = 0;
  args->v = 0;

  c = getopt(argc, argv, optString);
  while(c != -1){
    switch(c){
    case 's':                           /* sequence to be extracted */
      args->s = optarg;
      break;
    case 'c':                           /* print complement */
      args->c = 1;
      break;
    case 'h':                           /* print help */
      args->h = 1;
      break;
    case 'v':                           /* print version */
      args->v = 1;
      break;
    default:
      printf("# unknown argument: %c\n",c);
      args->e = 1;
      return args;
    }
    c = getopt(argc, argv, optString);
  }
  if(args->s == NULL && !args->v)
    args->e = 1;
  args->inputFiles = argv + optind;
  args->numInputFiles = argc - optind;

  return args;
}


void printUsage(char *version){
  printf("Purpose: get sequences from sequence file in FASTA format\n");
  printf("Usage: getSeq [options] [inputFile(s)]\n");
  printf("Options\n");
  printf("\t-s <REGEX> get sequences with definition lines matching REGEX\n");
  printf("\t[-c complement]\n");
  printf("\t[-h print this help message]\n");
  printf("\t[-v print version of program]\n");
}

void printSplash(char *version){
  printf("getSeq %s\n",version);
  printf("Written by Bernhard Haubold\n");
  printf("Distributed under the GNU General Public License\n");
  printf("Please send bug reports to haubold@evolbio.mpg.de\n");
  exit(0);
}
