/***** shustring.c ************************************************
 * Description: Compute shortest unique substrings.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Fri Apr 15 13:08:35 2005.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>
#include <regex.h>
#include "stree_ukkonen.h"
#include "eprintf.h"
#include "interface.h"
#include "stree_strmat.h"
#include "shustring.h"
#include "sequenceData.h"
#include "stringUtil.h"
#include "hash.h"

int main(int argc, char *argv[]){
  char *version;
  Sequence *forwardStrand = NULL;
  Sequence *reverseStrand = NULL;
  FILE *fpin;
  SUFFIX_TREE tree;
  int c, i, numString;
  Args args;

  version = "1.0";
  setprogname("shustring");

  args = getArgs(argc, argv);
  /* pass args to suffix tree computations */
  setArgs(args);

  if(args.h == 1){ 
    printUsage(version);
    return 0; 
  }else if(args.e == 1){
    printUsage(version);
    return -1;
  }else if(args.p){
    printSplash(version);
    exit(0);
  }

  tree = stree_new_tree(128, 1, 1, 10);
  tree->range = (Range *)emalloc(sizeof(Range));
  if(args.i == NULL){
    fpin = stdin;
  }else{
    fpin = efopen(args.i,"r");
  }if(args.o != NULL){       
    tree->fpout = efopen(args.o,"w");
  }else{
    tree->fpout = stdout;
  }
    
  c = 0;
  if(args.a){
    runHash(args);
    exit(0);
  }
  
  if(args.i == NULL)
    printf("# Reading data from stdin & constructing suffix tree...\n");
  else
    printf("# Reading data from file & constructing suffix tree...\n");
  numString = 0;
  while((forwardStrand = getPermanentNextSequence(fpin)) != NULL){
    if((args.n || args.r) && !args.u)
      convertToAcgt(forwardStrand);
    strtoupper(forwardStrand->seq);
    stree_ukkonen_add_string(tree, forwardStrand->seq, forwardStrand->seq, strlen(forwardStrand->seq), numString++, forwardStrand->id);
    if(args.r){
      reverseStrand = revcomp(forwardStrand);
      stree_ukkonen_add_string(tree, reverseStrand->seq, reverseStrand->seq, strlen(reverseStrand->seq), numString++, reverseStrand->id);
    }
  }
  fclose(fpin);
  int_stree_set_idents(tree);
  if(args.i == NULL)
    printf("#                                                       done.\n");
  else
    printf("#                                                      done.\n");
  
  tree->minShulen = (int *)emalloc(numString*sizeof(int));
  tree->numShustring = (int *)emalloc(numString*sizeof(int));
  tree->shustring = (Shustring **)emalloc(numString*sizeof(Shustring *));
  tree->shustringHead = (Shustring **)emalloc(numString*sizeof(Shustring *));
  for(i=0;i<tree->nextslot;i++){
    if(args.M < 0)
      tree->minShulen[i] = INT_MAX;
    else
      tree->minShulen[i] = args.M;
    tree->numShustring[i] = 0;
    tree->shustring[i] = (Shustring *)emalloc(sizeof(Shustring));
    tree->shustringHead[i] = tree->shustring[i];
  }
  /*
   * Computations on the suffix tree.
   */
  tree->value = (Value *)emalloc(stree_get_num_nodes(tree)*sizeof(Value));
  stree_traverse(tree,stringDepth,NULL);
  /* local shustrings */
  if(args.l != NULL){
    setLocalIndex(tree);
    if(args.s)
      stree_traverse(tree,collectLocalShustrings,NULL);
    else{
      if(args.q == 2)
	fprintf(tree->fpout,"# seqId\tseqlen\tnumshu\tshulen\n");
      /* traverse tree once for each string with local shustrings returned */
      for(i=0;i<tree->nextslot;i++){
	if(tree->localIndex[i]){
	  tree->localStrid = i;
	  fprintf(tree->fpout,"# ");
	  fprintnf(tree->fpout,tree->strnames[i],NAMELENGTH);
	  if(args.r && i % 2 == 1)
	    fprintf(tree->fpout,"-R");
	  if(args.q < 2)
	    fprintf(tree->fpout,"\n# num\tpos\tshulen");
	  if(args.q == 0){
	    fprintf(tree->fpout,"\tseq");
	  }
	  if(args.q < 2)
	    fprintf(tree->fpout,"\n");
	  tree->shuid = 0;
	  tree->range->min = INT_MAX;
	  tree->range->max = INT_MIN;
	  stree_traverse(tree,printLocalShustrings,NULL);
	  if(args.q < 2){
	    if(tree->range->min == tree->range->max)
	      fprintf(tree->fpout,"# L=%d\tn=%d\t%d\n",tree->lengths[i],tree->shuid,tree->range->max);
	    else
	      fprintf(tree->fpout,"# L=%d\tn=%d\t%d<=l<=%d\n",tree->lengths[i],tree->shuid,tree->range->min,tree->range->max);
	  }else if(args.q == 2){
	    if(tree->range->min == tree->range->max)
	      fprintf(tree->fpout,"\t%d\t%d\t%d\n",tree->lengths[i],tree->shuid,tree->range->max);
	    else{
	      fprintf(tree->fpout,"\t%d\t%d\t%d<=l<=%d\n",tree->lengths[i],tree->shuid,tree->range->min,tree->range->max);
	    }
	  }
	}
	
      }
    }
  }else{
    if(args.M > 0){
      for(i=0;i<numString;i++)
	tree->minShulen[i] = args.M;
    }
    stree_traverse(tree,findGlobalShustrings,NULL);
  }
  /* output */
  if(args.l == NULL || args.s)
    outputShustrings(tree);
  fclose(tree->fpout);
  if(args.o != NULL)
    printf("# Results written to file %s\n",args.o);
  return 0; 
}
